use crate::error::{Error, Result};
use askama_escape::{Escaper, MarkupDisplay};
use serde::Serialize;

/// Serialize to YAML (requires `serde_yaml` feature)
///
/// ## Errors
///
/// This will panic if `S`'s implementation of `Serialize` decides to fail,
/// or if `T` contains a map with non-string keys.
pub fn yaml<E: Escaper, S: Serialize>(e: E, s: S) -> Result<MarkupDisplay<E, String>> {
    match serde_yaml::to_string(&s) {
        Ok(s) => Ok(MarkupDisplay::new_safe(s, e)),
        Err(e) => Err(Error::from(e)),
    }
}

#[cfg(test)]
mod tests {
    use super::*;
    use askama_escape::Html;

    fn strip_document_seperator(s: &str) -> &str {
        // serde-yaml 0.8 includes a yaml "document seperator" at the start of
        // the encoded yaml, this function strips it out if present so the
        // tests pass with both serde-yaml 0.8 and 0.9
        if s.starts_with("---\n") {
            return &s[4..];
        } else {
            return s;
        }
    }

    #[test]
    fn test_yaml() {
        assert_eq!(strip_document_seperator(&yaml(Html, true).unwrap().to_string()), "true\n");
        assert_eq!(strip_document_seperator(&yaml(Html, "foo").unwrap().to_string()), "foo\n");
        assert_eq!(strip_document_seperator(&yaml(Html, true).unwrap().to_string()), "true\n");
        assert_eq!(strip_document_seperator(&yaml(Html, "foo").unwrap().to_string()), "foo\n");
        assert_eq!(
            strip_document_seperator(&yaml(Html, &vec!["foo", "bar"]).unwrap().to_string()),
            "- foo\n- bar\n"
        );
    }
}
